@@ -1,4 +1,5 @@
 #' @importFrom R6 R6Class
+#' @importFrom checkmate assert_vector assert_string assert_scalar
 #' @include impl-idd.R
 NULL
 
@@ -88,7 +89,8 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
         #'
         initialize = function (path) {
             # add a uuid
-            private$m_uuid <- unique_id()
+            private$m_log <- new.env(hash = FALSE, parent = emptyenv())
+            private$m_log$uuid <- unique_id()
 
             idd_file <- parse_idd_file(path)
             private$m_version <- idd_file$version
@@ -480,10 +482,19 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
         #' be further easily extracted using `$objects_in_relation()` method
         #' described below.
         #'
-        #' @param class A single integer specifying the class index or a single
+        #' @param which A single integer specifying the class index or a single
         #'        string specifying the class name.
         #' @param direction The relation direction to extract. Should be one of
         #'        `"all"`, `"ref_to"` or `"ref_by"`.
+        #' @param class A character vector of class names used for searching
+        #'        relations. Default: `NULL`.
+        #' @param group A character vector of group names used for searching
+        #'        relations. Default: `NULL`.
+        #' @param depth If > 0, the relation is searched recursively. A
+        #'        simple example of recursive reference: one material named
+        #'        `mat` is referred by a construction named `const`, and `const`
+        #'        is also referred by a surface named `surf`. If `NULL`,
+        #'        all possible recursive relations are returned. Default: `0`.
         #'
         #' @return An `IddRelation` object, which is a list of 3
         #' [data.table::data.table()]s named `ref_to` and `ref_by`.
@@ -498,8 +509,8 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
         #' idd$object_relation("Construction", "ref_by")
         #' }
         #'
-        object_relation = function (class, direction = c("all", "ref_to", "ref_by"))
-            idd_object_relation(self, private, class, match.arg(direction)),
+        object_relation = function (which, direction = c("all", "ref_to", "ref_by"), class = NULL, group = NULL, depth = 0L)
+            idd_object_relation(self, private, which, match.arg(direction), class = class, group = group, depth = depth),
         # }}}
 
         # objects_in_relation {{{
@@ -521,10 +532,25 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
         #' named list of an [IddObject] object named `Construction` and also all
         #' other [IddObject] objects that `Construction` can refer to.
         #'
-        #' @param class A single integer specifying the class index or a single
+        #' @param which A single integer specifying the class index or a single
         #'        string specifying the class name.
         #' @param direction The relation direction to extract. Should be either
         #'        `"ref_to"` or `"ref_by"`.
+        #' @param class A character vector of valid class names in the
+        #'        current Idd. It is used to restrict the classes to be
+        #'        returned. If `NULL`, all possible classes are considered and
+        #'        corresponding [IddObject] objects are returned if
+        #'        relationships are found. Default: `NULL`.
+        #' @param group A character vector of valid group names in the
+        #'        current Idd. It is used to restrict the groups to be
+        #'        returned. If `NULL`, all possible groups are considered and
+        #'        corresponding [IddObject] objects are returned if
+        #'        relationships are found. Default: `NULL`.
+        #' @param depth If > 0, the relation is searched recursively. A
+        #'        simple example of recursive reference: one material named
+        #'        `mat` is referred by a construction named `const`, and `const`
+        #'        is also referred by a surface named `surf`. If `NULL`,
+        #'        all possible recursive relations are returned. Default: `0`.
         #'
         #' @return An named list of [IddObject] objects.
         #'
@@ -537,8 +563,8 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
         #' idd$objects_in_relation("Construction", "ref_by")
         #' }
         #'
-        objects_in_relation = function (class, direction = c("ref_to", "ref_by"))
-            idd_objects_in_relation(self, private, class, match.arg(direction)),
+        objects_in_relation = function (which, direction = c("ref_to", "ref_by"), class = NULL, group = NULL, depth = 0L)
+            idd_objects_in_relation(self, private, which, match.arg(direction), class = class, group = group, depth = depth),
         # }}}
 
         # objects_in_group {{{
@@ -657,10 +683,18 @@ Idd <- R6::R6Class(classname = "Idd", cloneable = FALSE, lock_objects = FALSE,
 
     private = list(
         # PRIVATE FIELDS {{{
-        m_uuid = NULL,
         m_version = NULL,
         m_build = NULL,
-        m_idd_env = NULL
+        m_idd_env = NULL,
+        m_log = NULL,
+        # }}}
+
+        # PRIVATE FUNCTIONS {{{
+        uuid = function () private$m_log$uuid,
+        log_new_uuid = function () log_new_uuid(private$m_log),
+
+        idd_env = function () private$m_idd_env,
+        log_env = function () private$m_log
         # }}}
     )
 )
@@ -780,22 +814,26 @@ idd_objects <- function (self, private, class) {
 }
 # }}}
 # idd_object_relation {{{
-idd_object_relation <- function (self, private, class, direction = c("all", "ref_to", "ref_by")) {
-    assert(is_scalar(class))
+idd_object_relation <- function (self, private, which, direction = c("all", "ref_to", "ref_by"),
+                                 class = NULL, group = NULL, depth = 0L) {
+    assert_scalar(which)
     direction <- match.arg(direction)
 
-    cls <- get_idd_class(private$m_idd_env, class)
+    cls <- get_idd_class(private$m_idd_env, which)
 
     get_iddobj_relation(private$m_idd_env, cls$class_id, NULL, name = TRUE,
-        direction = direction, by_field = FALSE, max_depth = NULL, keep_all = FALSE
+        direction = direction, depth = depth, keep_all = FALSE,
+        class = class, group = group
     )
 }
 # }}}
 # idd_objects_in_relation {{{
-idd_objects_in_relation <- function (self, private, class, direction = c("ref_to", "ref_by")) {
-    assert(is_scalar(class))
+idd_objects_in_relation <- function (self, private, which, direction = c("ref_to", "ref_by"),
+                                     class = NULL, group = NULL, depth = 0L) {
+    assert_scalar(which)
     direction <- match.arg(direction)
-    rel <- get_idd_relation(private$m_idd_env, class, max_depth = 0L, direction = direction, keep_all = TRUE)
+    rel <- get_idd_relation(private$m_idd_env, which, depth = depth, direction = direction,
+        class = class, group = group, keep_all = TRUE)
 
     if (direction == "ref_to") {
         id_self <- unique(rel$class_id)
@@ -812,7 +850,7 @@ idd_objects_in_relation <- function (self, private, class, direction = c("ref_to
 
     if (!length(id_ref)) {
         dir <- switch(direction, ref_to = "does not refer to", ref_by = "is not referred by")
-        verbose_info("Class ", surround(cls_nm), " ", dir, " any other class.")
+        verbose_info("Class ", surround(cls_nm), " ", dir, " any other class or group.")
         return(obj_self)
     }
 
@@ -826,7 +864,7 @@ idd_objects_in_relation <- function (self, private, class, direction = c("ref_to
 # }}}
 # idd_objects_in_group {{{
 idd_objects_in_group <- function (self, private, group) {
-    assert(is_string(group))
+    assert_string(group)
 
     grp_id <- idd_group_index(self, private, group)
 
@@ -837,12 +875,6 @@ idd_objects_in_group <- function (self, private, group) {
     res
 }
 # }}}
-# idd_object_in_group {{{
-idd_object_in_group <- function (self, private, group) {
-    .deprecated_fun("$object_in_group()", "$objects_in_group()", "IdfObject", "0.10.0")
-    idd_objects_in_group(self, private, group)
-}
-# }}}
 # idd_to_table {{{
 idd_to_table <- function (self, private, class, all) {
     get_idd_table(private$m_idd_env, class, all)
@@ -864,28 +896,12 @@ idd_print <- function (self, private) {
 }
 # }}}
 
-#' @export
-# [.Idd {{{
-'[.Idd' <- function(x, i) {
-    if (!is.character(x)) return(NextMethod())
-
-    self <- ._get_self(x)
-    private <- ._get_private(x)
-
-    if (any(i %chin% private$m_idd_env$class$class_name)) {
-        .subset2(x, "objects")(i)
-    } else {
-        NextMethod()
-    }
-}
-# }}}
-
 #' @export
 # [[.Idd {{{
 `[[.Idd` <- function (x, i) {
     if (i %chin% ls(x)) return(NextMethod())
 
-    private <- ._get_private(x)
+    private <- get_priv_env(x)
 
     cls_id <- chmatch(i, private$m_idd_env$class$class_name)
 
@@ -902,9 +918,9 @@ idd_print <- function (self, private) {
 `$.Idd` <- function (x, i) {
     if (i %chin% ls(x)) return(NextMethod())
 
-    private <- ._get_private(x)
+    private <- get_priv_env(x)
 
-    cls_id <- chmatch(i, private$m_idd_env$class$class_name_us)
+    cls_id <- chmatch(underscore_name(i), private$m_idd_env$class$class_name_us)
 
     # skip if not a valid IDD class name
     if (is.na(cls_id)) return(NextMethod())
@@ -921,17 +937,48 @@ str.Idd <- function (object, ...) {
 }
 # }}}
 
+#' Format an Idd
+#'
+#' Format an [Idd] into a string.
+#'
+#' @param x An [Idd] object.
+#' @param ... Further arguments passed to or from other methods.
+#'
+#' @return A single length character vector.
+#' @examples
+#' \dontrun{
+#' format(use_idd(8.8, download = "auto"))
+#' }
+#'
 #' @export
 # format.Idd {{{
 format.Idd <- function (x, ...) {
-    paste0(
-        c(cli::rule("EnergyPlus Input Data Dictionary"),
-          paste0("Version", ": ", x$version()),
-          paste0("Build", ": ", x$build()),
-          paste0("Total Class", ": ", length(x$class_index()))
-        ),
-        collapse = "\n"
-    )
+    n <- length(x$class_index())
+
+    if (is.na(x$build())) {
+        sprintf("<EnergyPlus IDD v%s with %i %s>", x$version(), n,
+           if (n <= 1L) "class" else "classes"
+        )
+    } else {
+        sprintf("<EnergyPlus IDD v%s (%s) with %i classes>", x$version(), x$build(),
+            n, if (n <= 1L) "class" else "classes"
+        )
+    }
+}
+# }}}
+
+#' @export
+# ==.Idd {{{
+`==.Idd` <- function (e1, e2) {
+    if (!is_idd(e2)) return(FALSE)
+    identical(get_priv_env(e1)$uuid(), get_priv_env(e2)$uuid())
+}
+# }}}
+
+#' @export
+# !=.Idd {{{
+`!=.Idd` <- function (e1, e2) {
+    Negate(`==.Idd`)(e1, e2)
 }
 # }}}
 
@@ -1030,31 +1077,23 @@ read_idd <- function (path) {
 use_idd <- function (idd, download = FALSE) {
     if (is_idd(idd)) return(idd)
 
-    assert(is_scalar(idd))
+    assert_vector(idd, len = 1L)
 
     # if input is a file path or literal IDD string
     if (!is_idd_ver(idd)) {
-        return(tryCatch(read_idd(idd), error_read_file = function (e) {
-            abort("error_invalid_idd_input",
-                paste0("Parameter `idd` should be a valid version, a path, or ",
-                    "a single character string of an EnergyPlus Input Data ",
-                    "Dictionary (IDD) file (usually named `Energy+.idd`). ",
-                    "Invalid input found: ",
-                    if (length(idd) > 1L) {
-                        surround(paste0(idd[1L], "..."))
-                    } else {
-                        surround(idd)
-                    },
-                    "."
-                )
-            )
+        return(tryCatch(read_idd(idd), eplusr_error_read_lines = function (e) {
+            abort(paste0("Parameter 'idd' should be a valid version, a path, or ",
+                "a single character string of an EnergyPlus Input Data ",
+                "Dictionary (IDD) file (usually named 'Energy+.idd'). ",
+                "Invalid input found: ", str_trunc(surround(idd)), "."
+            ), "read_lines")
         }))
     }
 
     # make sure to print multiple version message only once
     ver_in <- standardize_ver(idd, complete = FALSE)
     # first check if version already exists
-    ver <- match_minor_ver(ver_in, c(names(.globals$idd), names(.globals$eplus_config)), "idd")
+    ver <- match_minor_ver(ver_in, c(names(.globals$idd), names(.globals$eplus)), "idd")
     # if not exists, try to find more
     if (is.na(ver)) ver <- match_minor_ver(ver_in, ALL_IDD_VER, "idd")
 
@@ -1067,12 +1106,12 @@ use_idd <- function (idd, download = FALSE) {
         if (is_avail_idd(ver)) return(.globals$idd[[as.character(ver)]])
 
         verbose_info("IDD v", ver, " has not been parsed before.\nTry to locate ",
-            "`Energy+.idd` in EnergyPlus v", ver, " installation folder ",
+            "'Energy+.idd' in EnergyPlus v", ver, " installation folder ",
             surround(eplus_default_path(ver)), ".")
 
         # if corresponding EnergyPlus folder not found
         if (!is_avail_eplus(ver)) {
-            verbose_info("Failed to locate `Energy+.idd` because EnergyPlus v",
+            verbose_info("Failed to locate 'Energy+.idd' because EnergyPlus v",
                 ver, " is not available.")
 
             # try to locate using latest IDFVersionUpdater
@@ -1087,13 +1126,11 @@ use_idd <- function (idd, download = FALSE) {
                     idd <- attr(dl, "file")
                 # else issue an error
                 } else {
-                    abort("error_no_matched_idd",
-                        paste0("Failed to locate IDD v", ver, ".\n",
-                            "You may want to set `download` to TRUE or ",
-                            "\"auto\" to download the IDD file from EnregyPlus ",
-                            "GitHub repo."
-                        )
-                    )
+                    abort(paste0("Failed to locate IDD v", ver, ".\n",
+                        "You may want to set 'download' to TRUE or ",
+                        "\"auto\" to download the IDD file from EnregyPlus ",
+                        "GitHub repo."
+                    ), "locate_idd")
                 }
             }
         # if corresponding EnergyPlus folder is found
@@ -1103,7 +1140,7 @@ use_idd <- function (idd, download = FALSE) {
 
             # but IDD file is missing
             if (!file.exists(idd)) {
-                verbose_info("`Energy+.idd` file does not exist in EnergyPlus v",
+                verbose_info("'Energy+.idd' file does not exist in EnergyPlus v",
                     config$version, " installation folder ", surround(config$dir), "."
                 )
 
@@ -1119,13 +1156,10 @@ use_idd <- function (idd, download = FALSE) {
                         idd <- attr(dl, "file")
                     # else issue an error
                     } else {
-                        abort("error_no_matched_idd",
-                            paste0("Failed to locate IDD v", ver,
-                                "You may want to set `download` to TRUE or ",
-                                "\"auto\" to download the IDD file from EnregyPlus ",
-                                "GitHub repo."
-                            )
-                        )
+                        abort(paste0("Failed to locate IDD v", ver,
+                            "You may want to set 'download' to TRUE or ",
+                            "\"auto\" to download the IDD file from EnregyPlus GitHub repo."
+                        ), "locate_idd")
                     }
                 }
             }
@@ -1144,8 +1178,10 @@ use_idd <- function (idd, download = FALSE) {
 #' @export
 # download_idd {{{
 download_idd <- function (ver = "latest", dir = ".") {
+    assert_vector(ver, len = 1L)
     ver <- standardize_ver(ver, complete = FALSE)
-    assert(is_scalar(ver), is_idd_ver(ver))
+
+    if (!is_eplus_ver(ver)) abort("'ver' must be a valid EnergyPlus version", "invalid_eplus_ver")
 
     ori_ver <- ver
     # if no patch version is given
@@ -1165,11 +1201,6 @@ download_idd <- function (ver = "latest", dir = ".") {
 
         if (ver == numeric_version("9.0.1")) ver <- numeric_version("9.0.0")
 
-    # in case explicitly download IDD version "9.0.0"
-    # because if ver is "9.0", then "9.0.1" will be downloaded, as there is no
-    # "9.0.0" IDD any more in current latest release.
-    } else if (ver == 9.0) {
-        latest_ver <- "9.0.0"
     # in case explicitly download IDD version "9.0.1"
     # change the file to download to "V9-0-0-Energy+.idd" as there is no
     # "V9-0-1-Energy+.idd" and only "V9-0-0-Energy+.idd", which does not follow
@@ -1202,8 +1233,7 @@ download_idd <- function (ver = "latest", dir = ".") {
     dest <- normalizePath(file.path(dir, file), mustWork = FALSE)
     res <- download_file(url, dest)
 
-    if (res != 0L)
-        stop(sprintf("Failed to download EnergyPlus IDD v%s.", ver), call. = FALSE)
+    if (res != 0L) abort(sprintf("Failed to download EnergyPlus IDD v%s.", ver), "download_idd")
 
     if (ver == latest_ver) {
         cmt <- ALL_EPLUS_RELEASE_COMMIT[version == ver][["commit"]]
@@ -1237,8 +1267,10 @@ avail_idd <- function () {
 #' @export
 # is_avail_idd {{{
 is_avail_idd <- function (ver) {
-    assert(is_idd_ver(ver, strict = TRUE))
-    !is.na(match_minor_ver(standardize_ver(ver, complete = FALSE), names(.globals$idd), "idd", verbose = FALSE))
+    if (is.character(ver) && "latest" %chin% ver) {
+        abort("'latest' notation for IDD version is not allowed here. Please give specific versions.", "ambiguous_idd_ver")
+    }
+    !is.na(convert_to_idd_ver(ver, strict = TRUE, all_ver = names(.globals$idd)))
 }
 # }}}
 
@@ -1247,7 +1279,7 @@ find_idd_from_updater <- function (ver) {
     ver <- standardize_ver(ver, strict = TRUE)
     # check if there are any EnergyPlus detected whose version is
     # newer than specified version
-    vers <- rev(avail_eplus()[avail_eplus() > ver])
+    vers <- rev(avail_eplus()[avail_eplus() >= ver])
 
     if (!length(vers)) return(NULL)
 
@@ -1260,7 +1292,7 @@ find_idd_from_updater <- function (ver) {
         line_break <- if (i == 1L) "" else "\n"
         dir <- file.path(eplus_config(vers[i])$dir, "PreProcess", "IDFVersionUpdater")
         idd_path <- normalizePath(file.path(dir, file), mustWork = FALSE)
-        msg <- paste0(msg, line_break, "Try to locate `", file, "` in EnergyPlus v",
+        msg <- paste0(msg, line_break, "Try to locate ", surround(file), " in EnergyPlus v",
             vers[i], " IDFVersionUpdater folder ", surround(dir), "."
         )
 
@@ -1289,25 +1321,21 @@ get_idd_from_ver <- function (idf_ver = NULL, idd = NULL, warn = TRUE) {
             # if input IDF has a version but neither that version of EnergyPlus
             # nor IDD is available, rewrite the message
             idd <- tryCatch(use_idd(idf_ver),
-                error_no_matched_idd = function (e) {
+                eplusr_error_locate_idd = function (e) {
                     mes <- stri_replace_all_fixed(conditionMessage(e),
-                        "You may want to set `download`",
-                        "You may want to use `use_idd()` and set `download`"
+                        "You may want to set 'download'",
+                        "You may want to use 'use_idd()' and set 'download'"
                     )
-                    abort("error_no_matched_idd", mes)
+                    abort(mes, "locate_idd")
                 }
             )
         } else {
             idd <- use_idd(idd)
             if (warn && idf_ver[, 1L:2L] != idd$version()[, 1L:2L]) {
-                warn("waring_idf_idd_mismatch_ver",
-                    paste0(
-                        "Version Mismatch. The IDF file parsing has a differnet ",
-                        "version (", idf_ver, ") than the IDD file used (",
-                        idd$version(), "). Parsing errors may occur."
-                    ),
-                    idf_ver = idf_ver,
-                    idd_ver = idd$version()
+                warn(paste0("Version mismatch. The IDF file parsing has a differnet ",
+                    "version (", idf_ver, ") than the IDD file used (",
+                    idd$version(), "). Parsing errors may occur."),
+                    "use_mismatch_idd"
                 )
             }
         }
@@ -1317,28 +1345,21 @@ get_idd_from_ver <- function (idf_ver = NULL, idd = NULL, warn = TRUE) {
         if (!is.null(idd)) {
             idd <- use_idd(idd)
             if (warn) {
-                warn("warning_given_idd_used",
-                    paste0(
-                        mes, " The given IDD version ", idd$version(),
-                        " will be used. Parsing errors may occur."
-                    )
+                warn(paste0(mes, " The given IDD version ", idd$version(),
+                    " will be used. Parsing errors may occur."), "use_hard_coded_idd"
                 )
             }
         } else {
             if (is.null(avail_idd())) {
-                abort("error_no_avail_idd",
-                    paste(mes, "No parsed IDD was available to use.")
-                )
+                abort(paste(mes, "No parsed IDD was available to use."), "no_avail_idd")
             }
 
             # which.max does not work with numeric_version objects
             idd <- use_idd(avail_idd()[max(order(avail_idd()))])
             if (warn) {
-                warn("warning_latest_idd_used",
-                    paste0(mes,
-                        " The latest parsed IDD version ", idd$version(),
+                warn(paste0(mes, " The latest parsed IDD version ", idd$version(),
                         " will be used. Parsing errors may occur."
-                    )
+                    ), "use_latest_idd"
                 )
             }
         }
