@@ -1,32 +1,89 @@
 context("IdfObject")
 
-# IdfObject class{{{
-test_that("IdfObject class", {
-    idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto"))
-    ver <- idf$Version
-    mat <- idf$Material$WD01
-    surf <- idf$BuildingSurface_Detailed[["WALL-1PF"]]
-    con <- idf$Construction[["WALL-1"]]
+use_idd(8.8, "auto")
 
-    # Basic {{{
-    # get parent Idf
-    expect_is(ver$parent(), "Idf")
+# NEW {{{
+test_that("$new()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
 
-    # get group name
-    expect_equal(con$group_name(), "Surface Construction Elements")
+    expect_error(IdfObject$new(1))
+    expect_error(IdfObject$new(1, class = "Material", parent = idf))
 
-    # get class name
-    expect_equal(con$class_name(), "Construction")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_equal(get_priv_env(mat)$m_object_id, 1L)
+    expect_equal(get_priv_env(mat)$m_class_id, 55L)
 
-    # get object ID
-    expect_equal(mat$id(), 1L)
-    # }}}
+    expect_is(mat <- IdfObject$new(1, 55, parent = idf), "IdfObject")
+    expect_equal(get_priv_env(mat)$m_object_id, 1L)
+    expect_equal(get_priv_env(mat)$m_class_id, 55L)
+})
+# }}}
+
+# VERSION {{{
+test_that("$version()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(5, parent = idf), "IdfObject")
+    expect_equal(obj$version(), numeric_version("8.8.0"))
+})
+# }}}
+
+# PARENT {{{
+test_that("$parent()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(5, parent = idf), "IdfObject")
+    expect_is(obj$parent(), "Idf")
+})
+# }}}
+
+# ID {{{
+test_that("$id()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_equal(obj$id(), 1L)
+})
+# }}}
+
+# NAME {{{
+test_that("$name()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_equal(obj$name(), "WD01")
+})
+# }}}
+
+# GROUP_NAME {{{
+test_that("$group_name()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(2, parent = idf), "IdfObject")
+    expect_equal(obj$group_name(), "Surface Construction Elements")
+})
+# }}}
+
+# CLASS_NAME {{{
+test_that("$class_name()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(2, parent = idf), "IdfObject")
+    expect_equal(obj$class_name(), "Construction")
+})
+# }}}
+
+# DEFINITION {{{
+test_that("$definition()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(obj <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(obj$definition(), "IddObject")
+})
+# }}}
+
+# COMMENT {{{
+test_that("$comment()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
 
-    # Comment {{{
     expect_equal(mat$comment(), " this is a test comment for WD01")
 
     # can handle invalid input types of comment
-    expect_error(mat$comment(comment = list("a")), class = "error_not_is.atomic")
+    expect_error(mat$comment(comment = list("a")))
 
     # can delete comments
     expect_equal(mat$comment(comment = NULL)$comment(), NULL)
@@ -38,25 +95,27 @@ test_that("IdfObject class", {
     expect_equal(mat$comment(comment = c("b"))$comment(), c("a", "b"))
 
     # can prepend comments
-    expect_equal(mat$comment(comment = c("c"), append = FALSE)$comment(),
-        c("c", "a", "b"))
+    expect_equal(mat$comment(comment = c("c"), append = FALSE)$comment(), c("c", "a", "b"))
 
     # can reset comments
     expect_equal(mat$comment(comment = c("d"), append = NULL)$comment(), "d")
 
     # can detect invalid `append` value
-    expect_error(mat$comment(comment = c("b"), append = 1:2),
-        class = "error_not_flag")
+    expect_error(mat$comment(comment = c("b"), append = 1:2), class = "eplusr_error")
 
     # can wrap comment at specified `width`
-    expect_equal(mat$comment(comment = c("a", "bb ccc"), append = NULL, width = 1L)$comment(),
-        c("a", "bb", "ccc"))
+    expect_equal(mat$comment(comment = c("a", "bb ccc"), append = NULL, width = 1L)$comment(), c("a", "bb", "ccc"))
 
     # can detect invalid `width` value
     expect_error(mat$comment(comment = c("a"), append = NULL, width = "a"))
-    # }}}
+})
+# }}}
+
+# VALUE {{{
+test_that("$value()", {
+    expect_is(idf <- read_idf(text("idf", 8.8)), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
 
-    # Get Values {{{
     # can handle cases when both `index` and `name` are NULL
     expect_equivalent(mat$value(), tolerance = 1e-5,
         list(Name = "WD01",
@@ -88,9 +147,9 @@ test_that("IdfObject class", {
         c("WD01", "MediumSmooth", "0.019099999", "0.115", "513", "1381", "0.9", "0.78", "0.78")
     )
 
-    # can detect invaid `index` values
-    expect_error(mat$value("1"), class = "error_bad_field_name")
-    expect_error(mat$value(c(1, 10:11)), class = "error_bad_field_index")
+    # can detect invalid `index` values
+    expect_error(mat$value("1"), class = "eplusr_error_invalid_field_name")
+    expect_error(mat$value(c(1, 10:11)), class = "eplusr_error_invalid_field_index")
 
     # can return subset of values in a object using `index`
     expect_equivalent(mat$value(c(3, 1, 5)), tolerance = 1e-5,
@@ -106,41 +165,43 @@ test_that("IdfObject class", {
     expect_equal(mat$Roughness, "MediumSmooth")
 
     # can detect invalid `name` values
-    expect_error(mat$value(c("Thickness", "Wrong", "Name")), class = "error_bad_field_name")
-    # }}}
+    expect_error(mat$value(c("Thickness", "Wrong", "Name")), class = "eplusr_error_invalid_field_name")
+})
+# }}}
+
+# SET {{{
+test_that("$set()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(ver <- IdfObject$new(5, parent = idf), "IdfObject")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(surf <- IdfObject$new(3, parent = idf), "IdfObject")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
 
-    # Set Values {{{
     # can stop when trying to directly modify `Version` object
-    expect_error(ver$set(8.8), class = "error_set_version")
+    expect_error(ver$set(8.8), class = "eplusr_error_set_version")
 
     # can stop when no values are given
-    expect_error(con$set(), class = "error_dot_empty")
+    expect_error(con$set(), class = "eplusr_error_validity_check")
 
-    expect_error(con$set(name = "named", "unnamed"),
-        class = "error_validity")
+    expect_error(con$set(name = "named", "unnamed"), class = "eplusr_error_validity_check")
 
     # can stop when duplicated names are given
-    expect_error(con$set(name = "first", name = "second"),
-        class = "error_dot_dup_field_name")
+    expect_error(con$set(name = "first", name = "second"))
 
     # can stop when invalid names are given for a non-extensible class
-    expect_error(mat$set(wrong = "something"),
-        class = "error_bad_field_name")
+    expect_error(mat$set(wrong = "something"), class = "eplusr_error_invalid_field_name")
 
     # can stop when invalid names are given for an extensible class
-    expect_error(con$set(name = "first", wrong = "second"),
-        class = "error_bad_field_name")
+    expect_error(con$set(name = "first", wrong = "second"), class = "eplusr_error_invalid_field_name")
 
     # can stop when valid names are given, but total field values are not accepatable for an extensible class
     idf$add(Zone = list("PLENUM-1"))
-    expect_error(surf$set(vertex_5_x_coordinate = 1, vertex_5_y_coordinate = 2),
-        class = "error_validity"
-    )
+    expect_error(surf$set(vertex_5_x_coordinate = 1, vertex_5_y_coordinate = 2), class = "eplusr_error_validity_check")
 
     # can stop when total field values are acceptable but invalid names are given for an extensible class
     expect_error(
         surf$set(vertex_5_x_coordinate = 1, vertex_5_y_coordinate = 2, vertex_5_z_wrong = 3),
-        class = "error_bad_field_name"
+        class = "eplusr_error_invalid_field_name"
     )
 
     # can add new values for extensible fields
@@ -153,7 +214,7 @@ test_that("IdfObject class", {
     expect_equal(con$value("Outside Layer")[[1]], "NewMaterialName")
 
     # can stop when there are invalid references in the input
-    expect_error(con$set(layer_2 = "missing"), class = "error_validity")
+    expect_error(con$set(layer_2 = "missing"), class = "eplusr_error_validity_check")
 
     # works using `[[<-.IdfObject`
     expect_silent(mat$Name <- "NewMaterial")
@@ -162,18 +223,23 @@ test_that("IdfObject class", {
     expect_equal(mat$name(), "NewMaterialName1")
     expect_silent(mat[[1]] <- "NewMaterialName")
     expect_equal(mat$name(), "NewMaterialName")
-    # }}}
+})
+# }}}
+
+# VALUE_POSSIBLE {{{
+test_that("$value_possible()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
 
-    # Possible {{{
     expect_equivalent(con$value_possible(),
         data.table(class_id = 90L, class_name = "Construction", object_id = 2L,
             object_name = "WALL-1", field_id = 11006:11010, field_index = 1:5,
             field_name = c("Name", "Outside Layer", paste("Layer", 2:4)),
-            value_id = 10:14, value_chr = c("WALL-1", "NewMaterialName", "PW03", "IN02", "GP01"),
+            value_id = 10:14, value_chr = c("WALL-1", "WD01", "PW03", "IN02", "GP01"),
             value_num = rep(NA_real_, 5),
             auto = NA_character_, default = rep(list(NA_character_), 5),
             choice = list(), range = rep(list(ranger(NA_real_, FALSE, NA_real_, FALSE)), 5),
-            source = c(list(NULL), rep(list(c("NewMaterialName", "WD02")), 4))
+            source = c(list(NULL), rep(list(c("WD01", "WD02")), 4))
         )
     )
     expect_equivalent(con$value_possible(6),
@@ -182,13 +248,94 @@ test_that("IdfObject class", {
             field_name = "Layer 5", value_id = -1L, value_chr = NA_character_, value_num = NA_real_,
             auto = NA_character_, default = list(NA_character_),
             choice = list(), range = list(ranger(NA_real_, FALSE, NA_real_, FALSE)),
-            source = list(c("NewMaterialName", "WD02"))
+            source = list(c("WD01", "WD02"))
         )
     )
-    # }}}
+})
+# }}}
+
+# VALIDATE {{{
+test_that("$validate()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+
+    expect_equal(con$validate()$invalid_reference,
+        data.table(object_id = 2L, object_name = "WALL-1", class_id = 90L,
+            class_name = "Construction", field_id = 11008:11010,
+            field_index = 3:5, field_name = paste("Layer", 2:4),
+            units = rep(NA_character_, 3L), ip_units = rep(NA_character_, 3L),
+            type_enum = 5L, value_id = 12:14,
+            value_chr = c("PW03", "IN02", "GP01"),
+            value_num = rep(NA_real_, 3L)
+        )
+    )
+
+    skip_on_cran()
+    if (!is_avail_eplus(8.8)) install_eplus(8.8)
+    idf <- read_idf(file.path(eplus_config(8.8)$dir, "ExampleFiles/5Zone_Transformer.idf"))
+    idf$dup(c(my_roof = "ROOF-1", "ROOF-1", "WALL-1"))
+    expect_equal(nrow(idf$validate()$invalid_reference), 0L)
+})
+# }}}
+
+# IS_VALID {{{
+test_that("$is_valid()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(ver <- IdfObject$new(5, parent = idf), "IdfObject")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(surf <- IdfObject$new(3, parent = idf), "IdfObject")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+
+    expect_true(ver$is_valid())
+    expect_true(mat$is_valid())
+    expect_false(con$is_valid())
+    expect_false(surf$is_valid())
+})
+# }}}
+
+# VALUE_RELATION {{{
+test_that("$value_relation()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
 
-    # Relation {{{
     expect_equivalent(con$value_relation(1),
+        list(
+            ref_to = data.table(
+                class_id = integer(), class_name = character(),
+                object_id = integer(), object_name = character(),
+                field_id = integer(), field_index = integer(), field_name = character(),
+                value_id = integer(), value_chr = character(), value_num = double(), type_enum = integer(),
+                src_class_id = integer(), src_class_name = character(),
+                src_object_id = integer(), src_object_name = character(),
+                src_field_id = integer(), src_field_index = integer(), src_field_name = character(),
+                src_value_id = integer(), src_value_chr = character(), src_value_num = double(), src_type_enum = integer(),
+                src_enum = integer(), dep = integer()
+            ),
+            ref_by = data.table(
+                class_id = 103L, class_name = "BuildingSurface:Detailed",
+                object_id = 3L, object_name = "WALL-1PF",
+                field_id = 11624L, field_index = 3L, field_name = "Construction Name",
+                value_id = 17L, value_chr = "WALL-1", value_num = NA_integer_, type_enum = 5L,
+                src_class_id = 90L, src_class_name = "Construction",
+                src_object_id = 2L, src_object_name = "WALL-1",
+                src_field_id = 11006L, src_field_index = 1L, src_field_name = "Name",
+                src_value_id = 10L, src_value_chr = "WALL-1", src_value_num = NA_integer_, src_type_enum = 4L,
+                src_enum = 2L, dep = 0L
+            ),
+            node = data.table(
+                class_id = integer(), class_name = character(),
+                object_id = integer(), object_name = character(),
+                field_id = integer(), field_index = integer(), field_name = character(),
+                value_id = integer(), value_chr = character(), value_num = double(), type_enum = integer(),
+                src_class_id = integer(), src_class_name = character(),
+                src_object_id = integer(), src_object_name = character(),
+                src_field_id = integer(), src_field_index = integer(), src_field_name = character(),
+                src_value_id = integer(), src_value_chr = character(), src_value_num = double(), src_type_enum = integer(),
+                src_enum = integer(), dep = integer()
+            )
+        )
+    )
+    expect_equivalent(con$value_relation(1, keep = TRUE),
         list(
             ref_to = data.table(
                 class_id = 90L, class_name = "Construction",
@@ -221,14 +368,36 @@ test_that("IdfObject class", {
                 src_object_id = 2L, src_object_name = "WALL-1",
                 src_field_id = 11006L, src_field_index = 1L, src_field_name = "Name",
                 src_value_id = 10L, src_value_chr = "WALL-1", src_value_num = NA_real_, src_type_enum = 4L,
-                src_enum = 2L, dep = 0L
+                src_enum = NA_integer_, dep = 0L
             )
         )
     )
+})
+# }}}
 
-    expect_equal(names(con$ref_to_object()), "NewMaterialName")
+# REF {{{
+test_that("$ref_to_object()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+    expect_is(ver <- IdfObject$new(5, parent = idf), "IdfObject")
+
+    expect_message(with_verbose(ver$ref_to_object()), "does not refer")
+    expect_message(with_verbose(ver$ref_to_object(class = "Material")), "does not refer")
+    expect_equal(names(con$ref_to_object()), "WD01")
+
+    expect_message(with_verbose(ver$ref_by_object()), "is not referred")
+    expect_message(with_verbose(ver$ref_by_object(class = "Material")), "is not referred")
     expect_equal(names(con$ref_by_object()), "WALL-1PF")
 
+    expect_message(with_verbose(ver$ref_to_node()), "has no node")
+    expect_message(with_verbose(ver$ref_to_node(class = "Material")), "has no node")
+ 
+    skip_on_cran()
+    if (!is_avail_eplus(8.8)) install_eplus(8.8)
+    idf <- read_idf(file.path(eplus_config(8.8)$dir, "ExampleFiles/5Zone_Transformer.idf"))
+    expect_is(loop <- IdfObject$new(278, parent = idf), "IdfObject")
+    expect_equal(length(loop$ref_to_node()), 9)
+
     expect_equal(con$has_ref(), c(TRUE, TRUE, FALSE, FALSE, FALSE))
     expect_true(con$has_ref(1))
     expect_equal(con$has_ref_to(), c(FALSE, TRUE, FALSE, FALSE, FALSE))
@@ -237,51 +406,41 @@ test_that("IdfObject class", {
     expect_equal(con$has_ref_by(), c(TRUE, rep(FALSE, 4)))
     expect_true(con$has_ref_by(1))
     expect_false(con$has_ref_by(2))
-    # }}}
+    expect_false(con$has_ref_node(2))
+})
+# }}}
 
-    # Validate {{{
-    expect_equal(con$validate()$invalid_reference,
-        data.table(object_id = 2L, object_name = "WALL-1", class_id = 90L,
-            class_name = "Construction", field_id = 11008:11010,
-            field_index = 3:5, field_name = paste("Layer", 2:4),
-            units = rep(NA_character_, 3L), ip_units = rep(NA_character_, 3L),
-            type_enum = 5L, value_id = 12:14,
-            value_chr = c("PW03", "IN02", "GP01"),
-            value_num = rep(NA_real_, 3L)
-        )
-    )
-    expect_true(ver$is_valid())
-    expect_true(mat$is_valid())
-    expect_false(con$is_valid())
-    expect_true(surf$is_valid())
-    # }}}
+# TO_TABLE {{{
+test_that("$to_table()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
 
-    # Table {{{
     expect_equal(
         con$to_table(all = FALSE, unit = TRUE, wide = FALSE, string_value = TRUE),
         data.table(id = 2L, name = "WALL-1", class = "Construction", index = 1:5,
             field = c("Name", "Outside Layer", paste("Layer", 2:4)),
-            value = c("WALL-1", "NewMaterialName", "PW03", "IN02", "GP01")
+            value = c("WALL-1", "WD01", "PW03", "IN02", "GP01")
         )
     )
     expect_equal(
         con$to_table(all = TRUE, unit = TRUE, wide = FALSE, string_value = TRUE),
         data.table(id = 2L, name = "WALL-1", class = "Construction", index = 1:11,
             field = c("Name", "Outside Layer", paste("Layer", 2:10)),
-            value = c("WALL-1", "NewMaterialName", "PW03", "IN02", "GP01", rep(NA_character_, 6))
+            value = c("WALL-1", "WD01", "PW03", "IN02", "GP01", rep(NA_character_, 6))
         )
     )
     expect_equal(
         con$to_table(all = TRUE, unit = FALSE, wide = FALSE, string_value = FALSE),
         data.table(id = 2L, name = "WALL-1", class = "Construction", index = 1:11,
             field = c("Name", "Outside Layer", paste("Layer", 2:10)),
-            value = as.list(c("WALL-1", "NewMaterialName", "PW03", "IN02", "GP01", rep(NA_character_, 6)))
+            value = as.list(c("WALL-1", "WD01", "PW03", "IN02", "GP01", rep(NA_character_, 6)))
         )
     )
     expect_equivalent(
         con$to_table(all = TRUE, unit = TRUE, wide = TRUE, string_value = FALSE),
         data.table(id = 2L, name = "WALL-1", class = "Construction",
-            Name = "WALL-1", `Outside Layer` = "NewMaterialName", `Layer 2` = "PW03",
+            Name = "WALL-1", `Outside Layer` = "WD01", `Layer 2` = "PW03",
             `Layer 3` = "IN02", `Layer 4` = "GP01", `Layer 5` = NA_character_,
             `Layer 6` = NA_character_, `Layer 7` = NA_character_, `Layer 8` = NA_character_,
             `Layer 9` = NA_character_, `Layer 10` = NA_character_
@@ -289,9 +448,9 @@ test_that("IdfObject class", {
     )
     expect_equivalent(tolerance = 1e-5,
         mat$to_table(string_value = FALSE),
-        data.table(id = 1L, name = "NewMaterialName", class = "Material", index = 1:9,
+        data.table(id = 1L, name = "WD01", class = "Material", index = 1:9,
             field = mat$definition()$field_name(),
-            value = list("NewMaterialName", "MediumSmooth", units::set_units(0.0191, m),
+            value = list("WD01", "MediumSmooth", units::set_units(0.0191, m),
                 units::set_units(0.115, W/K/m), units::set_units(513, kg/m^3),
                 units::set_units(1381, J/K/kg), 0.9, 0.78, 0.78
             )
@@ -299,7 +458,7 @@ test_that("IdfObject class", {
     )
     expect_equivalent(tolerance = 1e-5,
         mat$to_table(wide = TRUE, string_value = FALSE),
-        data.table(id = 1L, name = "NewMaterialName", class = "Material", Name = "NewMaterialName",
+        data.table(id = 1L, name = "WD01", class = "Material", Name = "WD01",
             `Roughness` = "MediumSmooth",
             `Thickness` = units::set_units(0.0191, m),
             `Conductivity` = units::set_units(0.115, W/K/m),
@@ -309,13 +468,18 @@ test_that("IdfObject class", {
             `Visible Absorptance` = 0.78
         )
     )
-    # }}}
+})
+# }}}
+
+# TO_STRING {{{
+test_that("$to_string()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
 
-    # String {{{
     expect_equal(con$to_string(leading = 0, sep_at = 10),
         c("Construction,",
           "WALL-1,   !- Name",
-          "NewMaterialName,  !- Outside Layer",
+          "WD01,     !- Outside Layer",
           "PW03,     !- Layer 2",
           "IN02,     !- Layer 3",
           "GP01;     !- Layer 4")
@@ -323,7 +487,7 @@ test_that("IdfObject class", {
     expect_equal(con$to_string(leading = 0, sep_at = 10, all = TRUE),
         c("Construction,",
           "WALL-1,   !- Name",
-          "NewMaterialName,  !- Outside Layer",
+          "WD01,     !- Outside Layer",
           "PW03,     !- Layer 2",
           "IN02,     !- Layer 3",
           "GP01,     !- Layer 4",
@@ -335,17 +499,72 @@ test_that("IdfObject class", {
           ";         !- Layer 10"
         )
     )
-    # }}}
+})
+# }}}
+
+# PRINT {{{
+test_that("$print()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(ver <- IdfObject$new(5, parent = idf), "IdfObject")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+
+    expect_output(ver$print())
+    expect_output(ver$print(brief = TRUE))
+    expect_output(con$print())
+    expect_output(con$print(all = TRUE), "11 :")
+    expect_output(con$print(auto_sep = FALSE))
+
+    expect_is(idf <- read_idf(example(), 8.8), "Idf")
+    expect_is(ver <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(out <- IdfObject$new(40, parent = idf), "IdfObject")
+    expect_output(ver$print())
+    expect_output(out$print())
+})
+# }}}
+
+# S3 FORMAT {{{
+test_that("format.IdfObject, as.character.IdfObject, etc", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+
+    expect_is(format(con), "character")
+    expect_equal(as.character(con), con$to_string())
+    expect_output(str(con))
+    expect_output(print(con))
+})
+# }}}
+
+# S3 SUBSET {{{
+test_that("$.IdfObject and [[.IdfObject", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+
+    expect_error(mat[1, 2])
+    expect_equal(mat[1:2], list(Name = "WD01", Roughness = "MediumSmooth"))
 
-    # S3 subsetting works {{{
     expect_equal(mat$Roughness, "MediumSmooth")
     expect_null(mat$rOuGhness)
+
     expect_equal(mat[["Roughness"]], "MediumSmooth")
     expect_null(mat[["roughness"]])
+    expect_error(mat[[1:2]])
+
+    expect_is(mat[["version"]], "function")
+})
+# }}}
+
+# S3 ASSIGN {{{
+test_that("$<-.IdfObject and [[<-.IdfObject", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+
+    expect_error(mat$name <- "Smooth")
     expect_silent(mat$Roughness <- "Smooth")
     expect_equal(mat$Roughness, "Smooth")
     expect_silent(mat[["Roughness"]] <- "MediumSmooth")
     expect_equal(mat[["Roughness"]], "MediumSmooth")
+    expect_silent(mat[[2]] <- "Rough")
+    expect_equal(mat[[2]], "Rough")
 
     expect_error(mat$roughness <- "Smooth", "cannot add bindings to")
     expect_error(mat[["roughness"]] <- "Smooth", "cannot add bindings to")
@@ -355,8 +574,57 @@ test_that("IdfObject class", {
     expect_silent(mat[["Visible Absorptance"]] <- 0.8)
     expect_equal(mat[["Visible Absorptance"]], 0.8, tolerance = 1e-5)
     expect_error(mat[["Visible_Absorptance"]] <- 0.8, "cannot add bindings to")
-    # }}}
+    expect_silent(mat[["Visible Absorptance"]] <- NULL)
+    expect_error(mat[[1:2]] <- 0.8)
+    expect_error(mat[["name"]] <- 0.8)
+})
+# }}}
 
-    expect_output(con$print())
+# S3 EQUITY {{{
+test_that("==.IdfObject and !=.IdfObject", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+    expect_is(mat <- IdfObject$new(1, parent = idf), "IdfObject")
+    expect_is(con <- IdfObject$new(2, parent = idf), "IdfObject")
+
+    expect_false(con == TRUE)
+    expect_true(con == con)
+    expect_false(con == mat)
+    expect_false(con != con)
+    expect_true(con != mat)
+})
+# }}}
+
+# IDF_OBJECT {{{
+test_that("idf_object()", {
+    expect_error(idf_object())
+
+    idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto"))
+
+    expect_error(idf_object(parent = idf))
+
+    expect_error(idf_object(idf, class = "Construction"))
+    expect_is(obj <- without_checking(idf_object(idf, class = "Construction")), "IdfObject")
+    expect_equal(obj$class_name(), "Construction")
+
+    expect_is(idf_object(idf, 5), "IdfObject")
+})
+# }}}
+
+# ACTIVE BINDING {{{
+test_that("add_idfobj_field_bindings()", {
+    expect_is(idf <- read_idf(text("idf", 8.8), use_idd(8.8, "auto")), "Idf")
+
+    expect_is(ver <- with_option(list(autocomplete = FALSE), add_idfobj_field_bindings(IdfObject$new(5, parent = idf))), "IdfObject")
+    expect_false("Version Identifier" %in% ls(ver))
+
+    expect_is(ver <- with_option(list(autocomplete = TRUE), add_idfobj_field_bindings(IdfObject$new(5, parent = idf))), "IdfObject")
+    expect_true("Version Identifier" %in% ls(ver))
+
+    expect_is(mat <- with_option(list(autocomplete = TRUE), add_idfobj_field_bindings(IdfObject$new(1, parent = idf))), "IdfObject")
+    expect_true(all(mat$definition()$field_name() %in% ls(mat)))
+    expect_silent(mat$Visible_Absorptance <- NULL)
+    expect_silent(mat <- add_idfobj_field_bindings(mat, update = TRUE))
+    expect_equal(length(mat$value()), 8)
+    expect_false("Visual Absorptance" %in% ls(mat))
 })
 # }}}
